# -*- coding: utf-8 -*-
"""
Created on Tue Apr 16 14:26:50 2024

The main file for Nmap
The script produces basic maps from static drivers of PALM runs

@author: Igor Ezau (NERSC, Norway)
"""
import numpy as np
import xarray
import cartopy.crs as ccrs

import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
from matplotlib.colors import LinearSegmentedColormap

#import custom packages
import OSM_map

plt.rcParams['figure.dpi'] = 300
plt.rcParams['savefig.dpi'] = 300

# type of palm output
palm_output_set = 'av_xy'
palm_domain_set = 'set0'
palm_period_set = '2019-07'
#palm_period_set = '2021-02'
palm_Nmap = 'Nmap'

## set0 - domain
data_location_folder = r'E:/(TURBAN) PALM runs DATA/Bergen_Scenarios/'
palm_scenario_folder = f'domain_{palm_domain_set}/{palm_period_set}/'
#
figure_folder = (data_location_folder + palm_scenario_folder + f'{palm_Nmap}/')

#
palm_data_file = (data_location_folder + palm_scenario_folder + 
                  f'combined_{palm_domain_set}_{palm_period_set}_{palm_output_set}.nc')
palm_scenario_static_driver = (data_location_folder + palm_scenario_folder + 
                               f'{palm_domain_set}_{palm_period_set}_static.nc')
#
palm_data_file_N02 = (data_location_folder + palm_scenario_folder + 
                  f'combined_{palm_domain_set}_{palm_period_set}_{palm_output_set}_N02.nc')
palm_scenario_static_driver_N02 = (data_location_folder + palm_scenario_folder + 
                               f'{palm_domain_set}_{palm_period_set}_static_N02.nc')

print(f'Creating basic map for the scenario {palm_domain_set}_{palm_period_set}')
print(f'Scenario combine dataset {palm_data_file}')
print(f'Scenario _static driver {palm_scenario_static_driver}')

#%% Reading the data - _static driver
sd = xarray.open_dataset(palm_scenario_static_driver, engine='netcdf4')
sd_N02 = xarray.open_dataset(palm_scenario_static_driver_N02, engine='netcdf4')
#sd = sd_N02 # only if child domain is in use
# reading PALM results
pr =xarray.open_dataset(palm_data_file, engine='netcdf4')
#pr_N02 =xarray.open_dataset(palm_data_file_N02, engine='netcdf4')
#pr = pr_N02

#%% Plot the basic map
# seting color maps
colors = ['lightblue','lightblue']
cm2_water = LinearSegmentedColormap.from_list("mycmap", colors)
colors = ['black','black']
cm2_building = LinearSegmentedColormap.from_list("mycmap", colors)
colors = ['blue','blue']
cm3_building = LinearSegmentedColormap.from_list("mycmap", colors)

# seting elevation levels for DEM
levels=np.linspace(20,660,33)

#%%
# selecting coordinate system
coordinate_system = 'WGS84' # options 'PALM', 'UTM', 'WGS84'

X, Y = 'lon','lat'
palm_domain = ([sd.lon[0][0],sd.lat[0][0]],
               [sd.lon[0][-1],sd.lat[0][-1]],
               [sd.lon[-1][-1],sd.lat[-1][-1]],
               [sd.lon[-1][0],sd.lat[-1][0]])

palm_N02_domain = ([sd_N02.lon[0][0],sd_N02.lat[0][0]],
                   [sd_N02.lon[0][-1],sd_N02.lat[0][-1]],
                   [sd_N02.lon[-1][-1],sd_N02.lat[-1][-1]],
                   [sd_N02.lon[-1][0],sd_N02.lat[-1][0]])

lon = 0.5*(sd.lon.max()+sd.lon.min())
lat = 0.5*(sd.lat.max()+sd.lat.min())
radius = max(sd.x.max(),sd.y.max())*0.75

#%%
# get background map
img = 'new'
fig, ax, data_crs, img = OSM_map.set_standard_map(lon,lat,sitename='Bergen',style='map',radius=radius, img = img)

for frame_time in range(len(pr['time'])):
#for frame_time in [0,1]:
#    pre_fix = 'tsurf'
#    pre_fix = 'theta_2m'
#    pre_fix = 'wspeed_10m'
    pre_fix = 'kc_PM10'
    # set variable pointer
#    sx = sd.assign(sat=pr[f'{pre_fix}*_xy'].isel(time=frame_time, zu1_xy=0))
    sx = sd.assign(pm10=(pr[f'{pre_fix}_xy']*1e4).isel(time=frame_time, zu_xy=3))
    frame_time_str = str(sx['time'].values)[0:16]
    frame_time_str = frame_time_str.replace(':','')
    print(f'Current time {frame_time_str}')
    figure_name = figure_folder + f'{pre_fix}/{palm_domain_set}_{pre_fix}_{frame_time_str}' + r'.png'
#    sx = sd.assign(sat=pr['theta_2m*_xy'].isel(time=frame_time, zu1_xy=0))
#    sx = sd.assign(ws=pr['wspeed_10m*_xy'].isel(time=frame_time, zu1_xy=0))
    # basemap
    fig, ax, data_crs, img = OSM_map.set_standard_map(lon,lat,sitename='Bergen',style='map',radius=radius, img = img)

    sd.zt.plot.contour(ax=ax, x=X, y=Y, 
                   levels=levels, linewidths=0.75, colors='k', alpha=1.0,
                   add_colorbar=False, transform=data_crs, zorder=2)

    cbar_kwargs = {'orientation':'vertical', 'shrink':0.6, 'aspect':40, 'label':'PM2.5 concentration [mkg/kg]'}
    sx.pm10.plot(ax=ax, x=X, y=Y,
            cmap='RdPu', alpha=0.8,
            cbar_kwargs=cbar_kwargs, add_colorbar=True, transform=data_crs, zorder=2)
#    cbar_kwargs = {'orientation':'vertical', 'shrink':0.6, 'aspect':40, 'label':'surface temperature [deg. C]'}
#    (sx-273.15).sat.plot(ax=ax, vmin=10, vmax=25, x=X, y=Y,
#    (sx-273.15).sat.plot(ax=ax, vmin=-12, vmax=-2, x=X, y=Y,
#    cbar_kwargs = {'orientation':'vertical', 'shrink':0.6, 'aspect':40, 'label':'wind speed at 10 m [m/s]'}
#    sx.sat.plot(ax=ax, x=X, y=Y,
#            cmap='coolwarm', alpha=0.8,
#            cbar_kwargs=cbar_kwargs, add_colorbar=True, transform=data_crs, zorder=2)

    sd.buildings_2d.plot.pcolormesh(ax=ax, x=X, y=Y,
                                cmap=cm2_building, 
                                add_colorbar=False, transform=data_crs, zorder=3)

    ax.add_patch(mpatches.Polygon(xy=palm_domain,
                              closed=True,
                              facecolor='none', edgecolor='k', linewidth=3,
                              transform=data_crs, zorder=4))

    sd_N02.buildings_2d.plot.pcolormesh(ax=ax, x=X, y=Y,
                                cmap=cm2_building, 
                                add_colorbar=False, transform=data_crs, zorder=3)

    ax.add_patch(mpatches.Polygon(xy=palm_N02_domain,
                              closed=True,
                              facecolor='none', edgecolor='b', linewidth=3,
                              transform=data_crs, zorder=4))
    
    ax.set_title(f'{palm_domain_set} {frame_time_str}')
    plt.show()
    #
    fig.savefig(figure_name, dpi=300)
print('--- The End ---')

